// -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/***
  BEGIN LICENSE

  Copyright (C) 2012 Mario Guerriero <mefrio.g@gmail.com>
  This program is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License version 3, as
  published    by the Free Software Foundation.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranties of
  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
  PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program.  If not, see <http://www.gnu.org/licenses>

  END LICENSE
***/

using Gtk;

namespace Wuuzy.Ui {

    public class WidgetsList : Gtk.TreeView {
        
        private Wuuzy.WidgetManager manager;
        
        private ListStore list_store;
        
        // Set objects to get selected widget
        private int count = 0;

        private Gee.HashMap<int, Wuuzy.WidgetObject> table;
        private Gee.HashMap<Wuuzy.WidgetObject, string> modules;
        private Gee.HashMap<string, Wuuzy.WidgetObject> objects;
        private string selected;
        
        public WidgetsList (Wuuzy.WidgetManager manager) {
            
            set_headers_visible (false);
            
            this.manager = manager;
            this.objects = new Gee.HashMap<string, Wuuzy.WidgetObject> ();
            this.table = new Gee.HashMap<int, Wuuzy.WidgetObject> ();
            this.modules = new Gee.HashMap<Wuuzy.WidgetObject, string> ();
            this.list_store = new ListStore (3, typeof (bool), typeof (string));
            
            set_model (list_store);
            
            // Set columns and renderers
            var toggle = new CellRendererToggle ();
            
            var column = new TreeViewColumn ();
            column.pack_start (toggle, false);
            column.add_attribute (toggle, "active", 0);
            append_column (column);
            
            var text = new CellRendererText ();
            
            column = new TreeViewColumn ();
            column.pack_start (text, true);
            column.add_attribute (text, "text", 1);
            append_column (column);
            
            toggle.activatable = true;
            toggle.toggled.connect ((path) => { 
                var tree_path = new TreePath.from_string (path);
                var selection = get_selection ();
                TreeIter iter;
                TreeModel model;
                if (selection.get_selected (out model, out iter)) {
                    list_store.get_iter (out iter, tree_path);
                    list_store.set (iter, 0, !toggle.active);
                    
                    // Get selected text
                    string sel = null;

                    GLib.Value val;
                    model.get_value (iter, 1, out val);
                    sel = val.dup_string ();
                    stdout.printf ("\nSelected %s\n", path);
                
                
                    var obj = table.get (int.parse (path));//objects.get (sel);
                    
                    //stdout.printf ("\nasdasd %s\n", obj.name);
                    /*var obj = table.get (int.parse (path));
                    var module = modules.get (obj);

                    var name = manager.get_object_name (obj);
                    obj = manager.get_object (name);
    stdout.printf ("\n%s was selected", obj.name);*/
                    if (toggle.active) {
                        manager.activate (obj, sel);
                    }
                    else {
                        manager.deactivate (obj, sel);
                    }
                }
            });
            
            this.get_selection ().set_mode (SelectionMode.BROWSE);
            
            // Populate
                        string widget_dir = Wuuzy.Constants.WIDGETDIR;
            string widget_name;
            GLib.Dir dir;
            
            try {
                dir = GLib.Dir.open (widget_dir);
            } catch (FileError e) {
			    stderr.printf ("%s\n", e.message);
			    return;
			}

            while ((widget_name = dir.read_name ()) != null) {
			  
			    string widget = widget_dir + "/" + widget_name;

			    if (GLib.FileUtils.test (widget, GLib.FileTest.EXISTS) && IOHelper.is_valid_file_type (widget)) {
				
				    if (!GLib.FileUtils.test (widget, GLib.FileTest.IS_DIR)) {
                
                        string path = Module.build_path (Wuuzy.Constants.WIDGETDIR, widget_name);
                        stdout.printf ("Loading plugin with path: '%s'\n", path);
                            
                        var module = Module.open (widget, ModuleFlags.BIND_LOCAL);

                        stdout.printf ("Loaded module: '%s'\n", module.name ());                        
                        
                        void* function;
                        module.symbol ("register_plugin", out function);
                        unowned RegisterWidgetFunction register_widget = (RegisterWidgetFunction) function;

                        var type = register_widget (module);
                        stdout.printf ("Plugin type: %s\n\n", type.name ());
                        
                        var obj = Object.new (type) as WidgetObject;
                        
                        TreeIter iter;
                        list_store.append (out iter);
                        list_store.set (iter, 0, manager.is_active (type.name ()), 1, obj.name);
                        
                        table.set (count, obj);
                        count++;
                        
                    }
               
                }
            }
            
        }
        
        public void append (Wuuzy.WidgetObject obj, bool active, string module_name) {

            TreeIter iter;
            list_store.append (out iter);
            list_store.set (iter, 0, active, 1, "a");

            // Setup objects to get selected widget
            //objects.set (obj.name, obj);
            //table.set (count, obj);
            modules.set (obj, module_name);   
            count++;
            
        }
     
    }

}