// -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/***
  BEGIN LICENSE

  Copyright (C) 2012 Mario Guerriero <mefrio.g@gmail.com>
  This program is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License version 3, as
  published    by the Free Software Foundation.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranties of
  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
  PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program.  If not, see <http://www.gnu.org/licenses>

  END LICENSE
***/

using Gtk;

using Wuuzy.Utils;

namespace Wuuzy.Ui {

    public class WuuzyWindow : Gtk.Window {
        
        private WuuzyUiApp app;

        // UI and Actions things
        const string ui_string = Wuuzy.Utils.MENUBAR;

        public Gtk.ActionGroup main_actions;
        public Gtk.UIManager ui;
        
        public WuuzyWindow (WuuzyUiApp app) {
            
            this.app = app;
            
            title = "Wuuzy";
            icon_name = ICON;
            set_default_size (400, 500);
            window_position = WindowPosition.CENTER;
            resizable = true;    
            
            // Init UI and Actions
            main_actions = new Gtk.ActionGroup ("MainActionGroup");
            main_actions.set_translation_domain ("wuuzy");
            main_actions.add_actions (main_entries, this);
            
            ui = new Gtk.UIManager ();

            try {
                ui.add_ui_from_string (ui_string, -1);
            }
            catch(Error e) {
                error ("Couldn't load the UI: %s", e.message);
            }

            Gtk.AccelGroup accel_group = ui.get_accel_group();
            add_accel_group (accel_group);

            ui.insert_action_group (main_actions, 0);
            ui.ensure_update ();

            // Setup window's ui
            setup_ui ();
            
        }
        
        private void setup_ui () {
            // Main box
            var box = new Box (Orientation.VERTICAL, 0);
            
            // MenuBar
            var menubar = ui.get_widget ("ui/MenuBar");
            
            // Central box
            var vbox = new Box (Orientation.VERTICAL, 12);
            
            var l = new Label (_("Widgets"));
            l.use_markup = true;
            l.label = "<span size='20000' color='#000000'>" + _("Widgets:") + "</span>";
            l.halign = Align.START;
            l.margin_left = 5;
            l.margin_top = 5;
            
            vbox.pack_start (l, false, false, 0);
            
            //var wview = new Wuuzy.Ui.WidgetsList (app.manager);
            //foreach (string name in app.manager.widgets.get_keys ()) {
            //foreach (string name in app.manager.get_widgets_names ()) {
            //var array = app.registrar.get_widgets_names();
            //for (int n=0; n<array.length; n++) {
            //    Wuuzy.WidgetObject obj = app.registrar.get_object (array[n]);
            //    bool active = app.manager.is_active (array[n]);
            //    wview.append (obj,  active, name);
            //}
            //wview.populate ();
            
            //var scrolled = new ScrolledWindow (null, null);
            //scrolled.add (wview);
            //scrolled.margin_left = 15;
            //scrolled.margin_right = 15;
            
            vbox.pack_start (manager.get_view (), true, true, 0);
            
            // Bottom bar
            var bottombar = new Toolbar ();
            bottombar.get_style_context ().add_class ("secondary-toolbar");
            
            l = new Label (_("Do you Like Wuuzy? Share it with your friends!"));
            
            var item = new ToolItem ();
            item.halign = Align.START;
            item.add (l);
            
            bottombar.add (item);
            
            var spacer = new ToolItem ();
            spacer.set_expand (true);
            
            bottombar.add (spacer);
            
            var im = new Image.from_icon_name (TWITTER_ICON, IconSize.SMALL_TOOLBAR);
            
            var b = new Button ();
            b.image = im;
            b.relief = ReliefStyle.NONE;
            b.clicked.connect (() => {
                try { Gtk.show_uri (null, TWITTER_URL, -1); }
                catch (Error e) { critical("Can't open the link"); }
            });
            
            item = new ToolItem ();
            item.halign = Align.END;
            item.add (b);
            
            bottombar.add (item);

            // Append everything to the window
            box.pack_start (menubar, false, false, 0);
            box.pack_start (vbox, true, true, 0);
            box.pack_start (bottombar, false, false, 0);
            
            this.add (box);
            
            this.show_all ();
        }
        
        // Actions functions
        void action_preferences () {
        
        }
        
        void action_close () {
            destroy ();
        }
        
        void show_about () {
            var about = new AboutDialog ();
            about.response.connect (() => { about.destroy (); });
            about.program_name = APP_TITLE;
            about.version = Wuuzy.Constants.VERSION;
            about.comments = COMMENTS;
            about.copyright = COPYRIGHTS;
            about.license = LICENSE;
            about.wrap_license = true;
            about.website = LAUNCHPAD_URL;
            about.authors = AUTHORS;
            about.logo_icon_name = ICON_ABOUT_LOGO;
            about.translator_credits = TRANSLATORS;
            about.run ();
        }
        
        void show_report () {
            try { Gtk.show_uri (null, BUG_URL, -1); }
            catch (Error e) { critical("Can't open the link"); }
        }

        void show_translate() {
            try { Gtk.show_uri (null, TRANSLATE_URL, -1); }
            catch (Error e) { critical("Can't open the link"); }
        }

        void show_app_help() {
            try { Gtk.show_uri (null, HELP_URL, -1); }
            catch (Error e) { critical("Can't open the link"); }
        }
        
        static const Gtk.ActionEntry[] main_entries = {
  /* name, stock id, label */  { "File", null, N_("_File") },
  /* name, stock id, label */  { "Help", null, N_("_Help") },
// Wuuzy actions
  /* name, stock id */         { "Preferences", Stock.PREFERENCES,
  /* label, accelerator */       N_("_Preferences"), null,
  /* tooltip */                  N_("Open Wuuzy's preferences"),
                                 action_preferences },
  /* name, stock id */         { "Close", Stock.CLOSE,
  /* label, accelerator */       N_("_Close"), "<control>Q",
  /* tooltip */                  N_("Close this folder"),
                                 action_close },
// Help actions
  /* name, stock id */         { "About", Stock.ABOUT,
  /* label, accelerator */       N_("_About"), null,
  /* tooltip */                  N_("Display credits"),
                                 show_about },
  /* name, stock id */         { "ReportProblem", "",
  /* label, accelerator */       N_("Report a Problem..."), null,
  /* tooltip */                  N_("File a bug on Launchpad"),
                                 show_report },
  /* name, stock id */         { "GetHelp", "",
  /* label, accelerator */       N_("Get Help Online..."), null,
  /* tooltip */                  "",
                                 show_app_help },
  /* name, stock id */         { "Translate", "",
  /* label, accelerator */       N_("Translate This Application..."), null,
  /* tooltip */                  "",
                                 show_translate }
        };
    
    }
    
}