// -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/***
  BEGIN LICENSE

  Copyright (C) 2012 Mario Guerriero <mefrio.g@gmail.com>
  This program is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License version 3, as
  published    by the Free Software Foundation.
plank
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranties of
  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
  PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program.  If not, see <http://www.gnu.org/licenses>

  END LICENSE
***/

using Gtk;

namespace Wuuzy {
    
    public class BaseWidget : GLib.Object {
        
        // Private Gtk widgets and other things
        private Window window;
        private CssProvider style_provider;
        private const string COMPOSITED_WINDOW_STYLESHEET = """
            .composited {
                background-color: rgba (0, 0, 0, 0.0);
            }
        """;
        
        public void init () {
            // Set up css provider
            style_provider = new CssProvider ();
            try {
                style_provider.load_from_data (COMPOSITED_WINDOW_STYLESHEET, -1);
            } catch (Error e) {
                warning ("Cannot load CSS: %s", e.message);
            }

            // Window properties
            window = new Window ();
            window.add_events (Gdk.EventMask.BUTTON_PRESS_MASK | Gdk.EventMask.POINTER_MOTION_MASK);
            window.motion_notify_event.connect (on_motion_notify);
            window.button_press_event.connect (on_window_button_press);
            window.set_keep_below (true);
            window.set_visual (window.get_screen ().get_rgba_visual());
            window.get_style_context().add_class("composited");
            window.get_style_context ().add_provider (style_provider, 600);
            window.app_paintable = true;
            window.decorated = false;
            window.stick ();        
        }
        
        public void run () {
            window.show_all ();
        }
        
        public void destroy () {
            int x, y;
            window.get_position (out x, out y);
            new GtkWidgets.PoofWindow (x + 100, y + 50); // FIXME
            window.destroy ();
        }
        
        public void add (Gtk.Widget widget) {
            window.add (widget);
        }
        
        private bool on_motion_notify (Gdk.EventMotion e) {
            window.get_window ().set_cursor (null);
            return true;
        }
        
        private bool on_window_button_press (Gdk.EventButton e) {
            window.begin_move_drag ((int)e.button, (int)e.x_root, (int)e.y_root, e.time);
            return true;
        }       
               
    }
}
