/*
 * Copyright (C) 2011-2012 Lucas Baudin <xapantu@gmail.com>
 *
 * Author: Zeeshan Ali (Khattak) <zeeshanak@gnome.org> (from Rygel)
 *
 * This file is part of Marlin.
 *
 * Marlin is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Marlin is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


namespace Wuuzy {

    public class Interface : Object {
        Manager manager;

        public enum Hook {
            WIDGET
        }

        public delegate void HookFunction ();
        public delegate void HookFunctionArg (Object object);
        
        //public unowned List<Wuuzy.TextView> all_source_view { private set; get; }

        public Interface (Manager manager) {
            this.manager = manager;

            manager.hook_widget.connect( (w) => {
                return;
            });

        }
        
        public void register_function (Hook hook, HookFunctionArg hook_function) {
            switch (hook) {
                case Hook.WIDGET:
                    manager.hook_widget.connect_after ((w) => {
                        hook_function(w);
                    });
                break;
            }
        }    
    }


    public class Manager : Object
    {

        public signal void hook_widget (Gtk.Widget widget);

        Peas.Engine engine;
        Peas.ExtensionSet exts;

        GLib.Settings settings;
        string settings_field;
        
        public Interface plugin_iface { private set; public get; }
        
        public Manager(GLib.Settings s, string f = "enabled-widgets", string dir = "/opt/wuuzy/widgets")
        {
            settings = s;
            settings_field = f;

            plugin_iface = new Interface (this);

            /* Let's init the engine */
            engine = Peas.Engine.get_default ();
            engine.enable_loader ("python");
            engine.enable_loader ("gjs");
            engine.add_search_path (dir, null);
            settings.bind(settings_field, engine, "loaded-plugins", SettingsBindFlags.DEFAULT);
            
            exts = new Peas.ExtensionSet (engine, typeof(Peas.Activatable), "object", plugin_iface, null);
            
            exts.extension_added.connect( (info, ext) => {  
                    ((Peas.Activatable)ext).activate();
            });
            exts.extension_removed.connect(on_extension_removed);
            
            exts.foreach (on_extension_added);
            
        }
        
        void on_extension_added(Peas.ExtensionSet set, Peas.PluginInfo info, Peas.Extension extension) {
            var core_list = engine.get_plugin_list ().copy ();
            for (int i = 0; i < core_list.length(); i++) {
                string module = core_list.nth_data (i).get_module_name ();
                if (module == info.get_module_name ())
                    ((Peas.Activatable)extension).activate();
                else
                    ((Peas.Activatable)extension).deactivate();
               stdout.printf ("\n%s", module);
            }
        }
        
        void on_extension_removed(Peas.PluginInfo info, Object extension) {
            ((Peas.Activatable)extension).deactivate();
        }
        
        public Gtk.Widget get_view () {
            var view = new PeasGtk.PluginManager (engine);
            var bottom_box = view.get_children ().nth_data (1) as Gtk.Box;
            assert(bottom_box != null);
            bottom_box.get_children ().nth_data(0).no_show_all = true;
            bottom_box.get_children ().nth_data(0).visible = false;
            return view;
        }

    }
}
