// -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/***
  BEGIN LICENSE

  Copyright (C) 2012 Mario Guerriero <mefrio.g@gmail.com>
  This program is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License version 3, as
  published    by the Free Software Foundation.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranties of
  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
  PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program.  If not, see <http://www.gnu.org/licenses>

  END LICENSE
***/

namespace Wuuzy {

    // Helper class for the file IO functions we'll need
    // Not needed at all, but helpful for organization
    public class IOHelper : GLib.Object {

        // Check if the filename has a picture file extension.
        public static bool is_valid_file_type (string fname, string suffix = ".so") {

            // Cache a lowe-cased copy of the file name
            string fname_down = fname.down();
            // Short-circuit if it's not a picture file extension
            return (fname_down.has_suffix(suffix));
        }

        // Quickly count up all of the valid widgets in the widget folder.
        public static GLib.List <weak KeyFile> count_widgets (GLib.File widget_folder) {

            GLib.FileInfo file_info = null;
            int count = 0;
            var files = new GLib.List <weak KeyFile> ();
            try {
                // Get an enumerator for all of the plain old files in the widget folder.
                var enumerator = widget_folder.enumerate_children(GLib.FileAttribute.STANDARD_NAME + "," + GLib.FileAttribute.STANDARD_TYPE, 0);
                // While there's still files left to count
                while ((file_info = enumerator.next_file ()) != null) {
                    // If it's a picture file
                    if (file_info.get_file_type() == GLib.FileType.REGULAR && is_valid_file_type(file_info.get_name(), ".widget")) {
                        string path = widget_folder.get_path () + "/" + file_info.get_name ();
                        var kfile = new GLib.KeyFile ();
                        try {
			                try {      
			                    kfile.load_from_file (path, GLib.KeyFileFlags.KEEP_COMMENTS | GLib.KeyFileFlags.KEEP_TRANSLATIONS);
			                } catch (FileError e) {
			                    stderr.printf ("%s\n", e.message);
			                }
			            } catch (KeyFileError e) {
			                stderr.printf ("%s\n", e.message);
			            }
                        files.append (kfile);
                        stdout.printf ("%s\nYeah\n%s\n", widget_folder.get_path (), file_info.get_name ());
                        count++;
                    }
                }
            } catch(GLib.Error err) {
                // Just for debugging sake
                stdout.printf("Could not pre-scan widget folder. Progress percentage may be off: %s\n", err.message);
            }
            return files;
        }

    }
    
    // Load all .widget files
    public class KeyFileManager : GLib.Object {
        
        public static GLib.List<GLib.KeyFile> load_from_dir (string dir_name) {
            
            var list = new GLib.List<weak GLib.KeyFile> ();
            
            string name;
            
            GLib.Dir dir = null;
            
            try {
                dir = GLib.Dir.open (dir_name);
            } catch (FileError e) {
                stderr.printf ("%s\n", e.message);
            }
            

            while ((name = dir.read_name ()) != null) {

			    string path = dir_name + name;
                
                GLib.KeyFile kfile = new GLib.KeyFile ();
                
			    if (GLib.FileUtils.test (path, GLib.FileTest.EXISTS) && IOHelper.is_valid_file_type (path, ".widget")) {

			        try {
			            try {      
			                kfile.load_from_file (path, GLib.KeyFileFlags.KEEP_COMMENTS);
			                list.append (kfile);
			            } catch (FileError e) {
			                stderr.printf ("%s\n", e.message);
			            }
			        } catch (KeyFileError e) {
			            stderr.printf ("%s\n", e.message);
			        }
			    }
			}
			return list;
        }
        
        public static string? get_value_from_file (KeyFile kfile, string value, string section = "Widget") {
            try {
                return kfile.get_value (section, value);
            } catch (KeyFileError e) {
                stderr.printf ("%s", e.message);
                return null;
            }
        }
        
        //public static GLib.KeyFile get_keyfile_from_name (string name) {
        
        //}
    
    }

}